#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Rick Spencer <rick.spencer@canonical.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
from desktopcouch.records.server import CouchDatabase
from desktopcouch.records.record import Record
from quickly.widgets.dictionary_grid import DictionaryGrid
from quickly.widgets.couch_grid import CouchGrid
from quickly.widgets.grid_filter import GridFilter
from quickly import prompts
from quickly.prompts import Prompt
from quickly.widgets import conventions
from quickly import prompts
from slip_cover.slip_coverconfig import get_data_path
from slip_cover.slip_coverconfig import design_doc
import gettext
from gettext import gettext as _
gettext.textdomain('slip-cover')
from slip_cover.ViewDesignerBox import ViewDesignerBox

class CouchDatabaseBox(gtk.VBox):
    def __init__(self, db_name):
        gtk.VBox.__init__(self, False, 5)

        self.db_name = db_name

        self.hpane = gtk.HPaned()
        self.hpane.set_property("position",300)
        self.hpane.show()
        self.pack_start(self.hpane, True, 5, 5 )

        parent_grid_container = gtk.VBox(False, 5)
        parent_grid_container.show()
        self.hpane.pack1(parent_grid_container, True, True)

        toolbar = gtk.Toolbar()
        toolbar.show()
        add_button = gtk.ToolButton(gtk.STOCK_ADD)
        add_button.show()
        add_button.connect("clicked",self.add_record_type)
        add_button.set_tooltip_text(_("Add record type"))
        toolbar.insert(add_button, -1)
        
        lbl1 = gtk.Label()
        lbl1.set_markup(_('<span weight="bold" underline="single">Record Types</span>'))
        lbl1.set_property("xalign",0)
        lbl1.show()
        parent_grid_container.pack_start(lbl1, False, False)
        parent_grid_container.pack_start(toolbar, False, False)
        

        rt_scroller = gtk.ScrolledWindow()
        rt_scroller.show()
        self.record_type_grid = DictionaryGrid({},keys=["Record Type","Deleted","Active"])
        self.record_type_grid.connect("selection-changed", self.record_type_selected)
        self.record_type_grid.show()
        rt_scroller.add(self.record_type_grid)
        parent_grid_container.pack_start(rt_scroller, True, True)

        self.database = CouchDatabase(db_name)
        if not self.database.view_exists("unique_rts", design_doc):
            map_js = """function(doc) {
return_obj = {"record_type":doc.record_type,"deleted":false}

for(var i in doc)
{
	if(i == "application_annotations")
	{
		if(doc[i]["Ubuntu One"])
		{
			if(doc[i]["Ubuntu One"]["private_application_annotations"])
			{
				if(doc[i]["Ubuntu One"]["private_application_annotations"]["deleted"])
				{
					return_obj["deleted"] = true;
				}
			}
		}
	}
}

emit(doc._id,return_obj);
            }"""
            self.database.add_view("unique_rts", map_js, None, design_doc)                         
        results = self.database.execute_view("unique_rts", design_doc)
        record_types = []
        for row in results:
            new = True
            for rt in record_types:
                if row.value["record_type"] == rt["Record Type"]:
                    if row.value["deleted"]:
                        rt["Deleted"] += 1
                    else:
                        rt["Active"] += 1
                    new = False
            if new:
                deletes = 0
                actives = 0
                if row.value["deleted"]:
                    deletes = 1
                else:
                    actives = 1
                d = {"Record Type":row.value["record_type"],"Deleted":deletes,"Active":actives,"editor":None}
                record_types.append(d)
        for rt in record_types:
            self.record_type_grid.append_row(rt)
        self.show()

        lbl2 = gtk.Label()
        lbl2.set_markup(_('<span weight="bold" underline="single">Views</span>'))
        lbl2.set_property("xalign",0)
        lbl2.show()
        parent_grid_container.pack_start(lbl2, False, False)

        toolbar = gtk.Toolbar()
        toolbar.show()
        add_button = gtk.ToolButton(gtk.STOCK_ADD)
        add_button.show()
        add_button.connect("clicked",self.add_view)
        add_button.set_tooltip_text(_("Add View"))
        toolbar.insert(add_button, -1)

        delete_button = gtk.ToolButton(gtk.STOCK_DELETE)
        delete_button.show()
        delete_button.connect("clicked",self.delete_views)
        delete_button.set_tooltip_text(_("Delete Selected Views"))
        toolbar.insert(delete_button, -1)


        parent_grid_container.pack_start(toolbar, False, False)
        
        self.views_grid = DictionaryGrid({},keys=["Design Document","View"])
        self.views_grid.connect("selection-changed", self.view_selected)
        self.views_grid.show()
        views_scroller = gtk.ScrolledWindow()
        views_scroller.show()
        views_scroller.add(self.views_grid)
        parent_grid_container.pack_start(views_scroller, True, True)

        views = []
        for i in list(self.database.db):
            if i.startswith("_design/"):
                doc = self.database.db[i]
                dd = i.split("/")[1]
                if "views" in doc:
                    for v in doc["views"]:
                        reduce_func = None
                        if "reduce" in doc["views"][v]:
                            reduce_func = doc["views"][v]["reduce"]
                        self.views_grid.append_row({"Design Document":dd,
                                                    "View":v,
                                                    "map":doc["views"][v]["map"],
                                                    "reduce":reduce_func,
                                                     "editor":None})
        self.editor_box = gtk.VBox(False, 5)
        self.editor_box.show()

        self.hpane.pack2(self.editor_box, True, True)

    def view_selected(self, widget, selected_rows, data=None):
        for c in self.editor_box.get_children():
            self.editor_box.remove(c)

        row = selected_rows[0]
        design_doc = row["Design Document"]
        view = row["View"]
        editor = row["editor"]
        map_func = row["map"]
        reduce_func = row["reduce"]
        if editor is None:
            editor = ViewDesignerBox(self.database,design_doc,view,
                                     map_func, reduce_func)
            editor.show()
            row["editor"] = editor

        self.editor_box.pack_start(editor, True, True)

    def add_view(self, widget, data=None):
        prompts.error("Slip Cover","Add View not yet implement")

    def delete_views(self, widget, data=None):
        for row in self.views_grid.selected_rows:
            title = _("Delete Views")
            message = _("""Are you sure you want to delete the view %s?
This action cannot be undone.""" % row["View"])
            yes = _("Delete View")
            no = _("Cancel")
            response = prompts.yes_no(title,message, yes, no)
            if response == gtk.RESPONSE_YES:
                print self.database.delete_view(row["View"],row["Design Document"])
        self.views_grid.remove_selected_rows()

    def record_type_selected(self, widget, selected_rows, data=None):
        for c in self.editor_box.get_children():
            self.editor_box.remove(c)
        
        row = selected_rows[0]
        if row["editor"] == None:
            row["editor"] = RecordsEditor(self.db_name,
                                            row["Record Type"])
        self.editor_box.pack_start(row["editor"])

    def add_record_type(self, widget, data=None):
        title = _("Add Record Type")
        msg = _("""
        Specify a string name for the record type. By convention, the
        name should be a URL where you can document the user of the record type.
        """)
        response, val = prompts.string(title,msg,"http://wiki.ubuntu.com/")
        if response == gtk.RESPONSE_OK:
            r = Record({"record_type":val})
            self.database.put_record(r)
            self.record_type_grid.append_row({"Record Type":val,"editor":None})

class RecordsEditor(gtk.VBox):
    def __init__(self, db_name, record_type):
        gtk.VBox.__init__(self, False, 5)

        self.db_name = db_name
        self.database = CouchDatabase(db_name)
        toolbar = gtk.Toolbar()
        delete_button = gtk.ToolButton(gtk.STOCK_DELETE)
        delete_button.set_tooltip_text(_("Delete selected records"))
        delete_button.show()
        delete_button.connect("clicked",self.delete_rows)
        toolbar.insert(delete_button, -1)

        add_button = gtk.ToolButton(gtk.STOCK_ADD)
        add_button.set_tooltip_text(_("Add record"))
        add_button.show()
        add_button.connect("clicked",self.add_row)
        toolbar.insert(add_button, -1)

        new_col_button = gtk.ToolButton()
        img = gtk.Image()
        img.set_from_file(get_data_path() + "/media/add_key.png")
        img.show()

        new_col_button.set_icon_widget(img)
        new_col_button.set_tooltip_text(_("Add key"))
        new_col_button.show()
        new_col_button.connect("clicked",self.add_key)
        toolbar.insert(new_col_button, -1)

        del_col_button = gtk.ToolButton()
        img = gtk.Image()
        img.set_from_file(get_data_path() + "/media/del_key.png")
        img.show()


        del_col_button.set_icon_widget(img)
        del_col_button.set_tooltip_text(_("Delete selected key"))
        del_col_button.show()
        del_col_button.connect("clicked",self.delete_keys)
        toolbar.insert(del_col_button, -1)

        refresh_button = gtk.ToolButton(gtk.STOCK_REFRESH)
        refresh_button.set_tooltip_text(_("Refresh"))
        refresh_button.show()
        refresh_button.connect("clicked",self.refresh)
        toolbar.insert(refresh_button, -1)

        toolbar.show()
        self.pack_start(toolbar, False, False)

        self.records_scroller = gtk.ScrolledWindow()
        self.records_scroller.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
        self.records_scroller.show()
        try:
            self.grid = CouchGrid(db_name,
                              record_type, editable=True)
        except RuntimeError:
            msg = _("""All of the records of record type %s have been
            deleted. Enter a new key and click OK if you want to edit records
            of this record type. Otherwise click Cancel.
            
            """ %record_type)
            response, val = prompts.string(_("Create Column"),msg)
            if response != gtk.RESPONSE_OK:
                raise RuntimeError(_("No records to display"))
            else:
                self.grid = CouchGrid(db_name, keys = [val],
                              record_type = record_type, editable=True)
        self.grid.show()

        self.filt = GridFilter(self.grid)
        self.filt.show()
        self.pack_start(self.filt, False, False)

        self.records_scroller.add_with_viewport(self.grid)
        self.pack_start(self.records_scroller, True, True)
        self.show()

    def refresh(self, widget, data=None):
        self.grid._refresh_treeview()
        self.remove(self.filt)
        self.filt = GridFilter(self.grid)
        self.pack_start(self.filt, False, False)
        self.reorder_child(self.filt,1)
        self.filt.show()

    def delete_rows(self, widget, data=None):
        for r in self.grid.selected_rows:
            if "__desktopcouch_id" in r:
                self.database.delete_record(r["__desktopcouch_id"])

        #remove the rows
        self.grid.remove_selected_rows()

    def add_row(self, widget, data=None):
        self.grid.append_row({})

    def add_key(self, widget, data=None):
        response, val = prompts.string(_("New Key"),_("Enter a name for the new key"))
        if response == gtk.RESPONSE_OK:
            self.grid.keys.append(val)
            self.grid._refresh_treeview()

    def delete_keys(self, widget, data=None):
        title = _("Delete Keys")
        message = _("""
        Choose keys to delete. Note that this will delete all values for these keys in 
        the database and this action cannot be undone.
        """)
        keys_to_delete = []
        val = {}
        for k in self.grid.keys:
            val[k] = False
        response, val = checklist(title, message, val)
        if response == gtk.RESPONSE_OK:
            msg = _("Are you certain you want to permanently delete the follow keys and values?")
            msg += "\n\n"
            t = False
            for k in val.keys():
                if val[k]:
                    keys_to_delete.append(k)
                    if t:
                        msg += ", "
                    t = True
                    msg += k

            response = prompts.yes_no("Delete Keys Warning", msg, "Delete Keys and Values", "Cancel")
            if response == gtk.RESPONSE_YES:
                self.grid.get_model().foreach(self.delete_keys_from_store,keys_to_delete)

                for k in keys_to_delete:
                    self.grid.keys.remove(k)
                self.grid._refresh_treeview()
                
    def delete_keys_from_store(self, model, path, iter, keys_to_delete):
        d = model.get_value(iter,len(self.grid.keys))
        for k in keys_to_delete:
            if k in d:
                del(d[k])
        if '__desktopcouch_id' in d:
            keys = d.keys()
            for k in keys:
                if k.startswith("__desktopcouch"):
                    dc_key = k.split("__desktopcouch")[1]
                    d[dc_key] = d[k]
                    del(d[k])
                if k == "__record_type":
                    d["record_type"] = d["__record_type"]
                    del(d["__record_type"])
            self.database.delete_record(d['_id'])
            del(d["_rev"])
            del(d["_id"])
            self.database.put_record(Record(d))

class CheckListPrompt(Prompt):
    def __init__(self, title = "Input String", text = "Input a String:", default_value = None):

        Prompt.__init__(self, title, text)
        if default_value is None:
            default_value = {}
        self._vbox = gtk.VBox(True,5)
        self._vbox.show()
        keys = sorted(default_value)
        for k in keys:
            cb = gtk.CheckButton(k)
            cb.set_active(default_value[k])
            cb.show()
            self._vbox.pack_start(cb)
        self.content_box.pack_end(self._vbox, True, True, 5)

    def get_value(self):
        return_dict = {}
        for c in self._vbox.get_children():
            return_dict[c.get_label()] = c.get_active()
        return return_dict

def checklist(title = "Input String", text = "Input a String:", default_value = None):
    clp = CheckListPrompt(title, text, default_value)
    response = clp.run()
    val = clp.get_value()
    clp.destroy()
    return (response, val)



