# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Rick Spencer <rick.spencer@canonical.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import gtksourceview2
from quickly import prompts

from slip_cover.helpers import get_builder
from slip_cover.slip_coverconfig import design_doc
from desktopcouch.records.server import CouchDatabase
import gettext
from quickly.widgets.dictionary_grid import DictionaryGrid
from gettext import gettext as _
gettext.textdomain('slip-cover')

class ViewDesignerBox(gtk.VBox):

    def __init__(self, database, design_doc=None, view_name=None,
                      map_func=None,reduce_func=None):
        gtk.VBox.__init__(self, False, 5)
        self.database = database
        self.design_doc = design_doc
        self.view_name = view_name

        toolbar = gtk.Toolbar()
        toolbar.show()
        run_button = gtk.ToolButton(gtk.STOCK_MEDIA_PLAY)
        run_button.set_tooltip_text(_("Run View"))
        run_button.show()
        run_button.connect("clicked",self.run_query)
        toolbar.insert(run_button, -1)

        save_button = gtk.ToolButton(gtk.STOCK_SAVE)
        save_button.set_tooltip_text(_("Run View"))
        save_button.show()
        save_button.connect("clicked",self.save_query)
        toolbar.insert(save_button, -1)

        splitter = gtk.VPaned()
        splitter.show()
        js_box = gtk.VBox(False, 5)
        js_box.show()

        self.map_func_scroller = gtk.ScrolledWindow()        
        self.map_func_scroller.show()

        self.map_buffer = gtksourceview2.Buffer()
        lang_man = gtksourceview2.language_manager_get_default()
        javascript = lang_man.get_language("js")
        self.map_buffer.set_language(javascript)
        self.map_buffer.set_highlight_syntax(True)
        if map_func is None:
            map_func = """function(doc)
{
    emit(doc._id,doc);
}
"""
        self.map_buffer.set_text(map_func)
        map_view = gtksourceview2.View(self.map_buffer)
        map_view.show()
        self.map_func_scroller.add(map_view)

        self.checkbutton_reduce = gtk.CheckButton()
        self.checkbutton_reduce.set_active(False)
        self.checkbutton_reduce.set_label(_("Use reduce function"))
        self.checkbutton_reduce.connect("toggled",self.use_reduce)
        self.checkbutton_reduce.show()

        self.reduce_func_scroller = gtk.ScrolledWindow()
        self.reduce_buffer = gtksourceview2.Buffer()
        lang_man = gtksourceview2.language_manager_get_default()
        javascript = lang_man.get_language("js")
        self.reduce_buffer.set_language(javascript)
        self.reduce_buffer.set_highlight_syntax(True)
        if reduce_func is None:
            reduce_func = """function(keys,values,rereduce)
{
    return values.length;
}
"""
        else:
            self.checkbutton_reduce.set_active(True)
            self.reduce_func_scroller.show()

        self.reduce_buffer.set_text(reduce_func)
        reduce_view = gtksourceview2.View(self.reduce_buffer)
        reduce_view.show()
        self.reduce_func_scroller.add(reduce_view)

        self.results_scroller = gtk.ScrolledWindow()
        self.results_scroller.show()

        self.pack_start(toolbar, False, False)
        self.pack_start(splitter, True, True)
        splitter.pack1(js_box, True, True)
        js_box.pack_start(self.map_func_scroller,True,True)
        js_box.pack_start(self.checkbutton_reduce, False, False)
        js_box.pack_start(self.reduce_func_scroller, True, True)
        splitter.pack2(self.results_scroller, True, True)

    def run_query(self, widget, data=None):
        start_iter = self.map_buffer.get_iter_at_offset(0)
        end_iter =  self.map_buffer.get_iter_at_offset(-1)
        map_func = self.map_buffer.get_text(start_iter, end_iter)

        reduce_func = None
        if self.checkbutton_reduce.get_active():
            start_iter = self.reduce_buffer.get_iter_at_offset(0)
            end_iter =  self.reduce_buffer.get_iter_at_offset(-1)
            reduce_func = self.reduce_buffer.get_text(start_iter, end_iter)

        try:
            results = self.database._temporary_query(map_func, reduce_func)
            dicts = []
            for r in results.rows:
                d = {}
                if r.key:
                    d["key"] = r.key
                if type(r.value) is dict:
                    for k in r.value.keys():
                        d[k] = r.value[k]
                else:
                    d["value"] = r.value
                dicts.append(d)

            self.grid = DictionaryGrid(dicts)
            self.grid.show()
            for c in self.results_scroller.get_children():
                self.results_scroller.remove(c)
            self.results_scroller.add(self.grid)
        except Exception, inst:
            title = _("Error running map/reduce")
            prompts.error(title,inst.message)

    def save_query(self, widget, data=None):
        prompts.error("Slip Cover",_("Save Not Yet Implemented"))

    def use_reduce(self, widget, data=None):
        display = widget.get_active()
        self.reduce_func_scroller.set_visible(display)

